# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.utils.translation import ugettext_lazy as _
from django_fsm import transition
import stripe
from shop.models.order import BaseOrder, OrderPayment
from shop.money import MoneyMaker
from shop_stripe.payment import StripePayment


class OrderWorkflowMixin(object):
    TRANSITION_TARGETS = {
        'paid_with_stripe': _("Paid using Stripe"),
    }

    def __init__(self, *args, **kwargs):
        if not isinstance(self, BaseOrder):
            raise ImproperlyConfigured("class 'OrderWorkflowMixin' is not of type 'BaseOrder'")

        super(OrderWorkflowMixin, self).__init__(*args, **kwargs)

    @transition(field='status', source=['created'], target='paid_with_stripe')
    def add_stripe_payment(self, charge):
        assert self.currency == charge['currency'].upper(), "Currency mismatch"
        Money = MoneyMaker(self.currency)
        amount = Money(charge['amount']) / Money.subunits
        OrderPayment.objects.create(
            order=self,
            amount=amount,
            transaction_id=charge['id'],
            payment_method=StripePayment.namespace,
        )

    def is_fully_paid(self):
        return super(OrderWorkflowMixin, self).is_fully_paid()

    @transition(field='status', source='paid_with_stripe', conditions=[is_fully_paid],
                custom=dict(admin=True, button_name=_("Acknowledge Payment")))
    def acknowledge_stripe_payment(self):
        self.acknowledge_payment()

    def refund_payment(self):
        """
        Refund the payment using Stripe's refunding API.
        """
        Money = MoneyMaker(self.currency)
        filter_kwargs = {
            'transaction_id__startswith': 'ch_',
            'payment_method': StripePayment.namespace,
        }
        for payment in self.orderpayment_set.filter(**filter_kwargs):
            refund = stripe.Refund.create(charge=payment.transaction_id)
            if refund['status'] == 'succeeded':
                amount = Money(refund['amount']) / Money.subunits
                OrderPayment.objects.create(
                    order=self,
                    amount=-amount,
                    transaction_id=refund['id'],
                    payment_method=StripePayment.namespace,
                )

        del self.amount_paid  # to invalidate the cache
        if self.amount_paid:
            # proceed with other payment service providers
            super(OrderWorkflowMixin, self).refund_payment()

    def cancelable(self):
        return super(OrderWorkflowMixin, self).cancelable() or self.status in ['paid_with_stripe']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.utils.text import format_lazy
from django.utils.translation import ugettext_lazy as _
from rest_framework.exceptions import ValidationError
import stripe
from shop.models.order import OrderModel
from shop.payment.providers import PaymentProvider

stripe.api_key = settings.SHOP_STRIPE['APIKEY']


class StripePayment(PaymentProvider):
    """
    Provides a payment service for Stripe.
    """
    namespace = 'stripe-payment'

    def get_payment_request(self, cart, request):
        """
        From the given request, add a snippet to the page.
        """
        try:
            order = self.charge(cart, request)
            js_expression = 'window.location.href="{}";'.format(order.get_absolute_url())
            return js_expression
        except stripe.error.StripeError as err:
            raise ValidationError(err._message)
        except KeyError as err:
            raise ValidationError(str(err))

    def charge(self, cart, request):
        """
        Use the Stripe token from the request and charge immediately.
        This view is invoked by the Javascript function `scope.charge()` delivered
        by `get_payment_request`.
        """
        token_id = cart.extra['payment_extra_data'].get('token_id')
        if not token_id:
            message = _("Stripe payment token is missing")
            raise stripe.error.StripeError(message)

        order = OrderModel.objects.create_from_cart(cart, request)
        charge = stripe.Charge.create(
            amount=cart.total.as_integer(),
            currency=cart.total.currency,
            source=token_id,
            transfer_group=order.get_number(),
            description=settings.SHOP_STRIPE['PURCHASE_DESCRIPTION'],
        )
        if charge['status'] == 'succeeded':
            order.populate_from_cart(cart, request)
            order.add_stripe_payment(charge)
            order.save(with_notification=True)
        else:
            message = _("Stripe returned status '{status}' for id: {id}")
            raise stripe.error.StripeError(format_lazy(message, **charge))
        return order



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import date
from django.utils.translation import ugettext_lazy as _
from shop.payment.modifiers import PaymentModifier
from .payment import StripePayment


class StripePaymentModifier(PaymentModifier):
    payment_provider = StripePayment()
    commision_percentage = None

    def get_choice(self):
        return (self.identifier, _("Credit Card"))

    def is_disabled(self, cart):
        return cart.total == 0

    def add_extra_cart_row(self, cart, request):
        from decimal import Decimal
        from shop.serializers.cart import ExtraCartRow

        if not self.is_active(cart) or not self.commision_percentage:
            return
        amount = cart.total * Decimal(self.commision_percentage / 100.0)
        instance = {'label': _("+ {}% handling fee").format(self.commision_percentage), 'amount': amount}
        cart.extra_rows[self.identifier] = ExtraCartRow(instance)
        cart.total += amount

    def update_render_context(self, context):
        super(StripePaymentModifier, self).update_render_context(context)
        today = date.today()
        context['payment_modifiers']['month_range'] = \
            [(date(2000, m, 1).strftime('%m'), date(2000, m, 1).strftime('%b')) for m in range(1, 13)]
        context['payment_modifiers']['years_range'] = range(today.year, today.year + 11)
        context['payment_modifiers']['stripe_payment'] = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings


def public_keys(request):
    return {
        'SHOP_STRIPE_PUBKEY': settings.SHOP_STRIPE['PUBKEY'],
        'SHOP_STRIPE_PREFILL': getattr(settings, 'SHOP_STRIPE_PREFILL', False)
    }



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from setuptools import setup, find_packages
import shop_stripe
try:
    from pypandoc import convert
except ImportError:
    import io

    def convert(filename, fmt):
        with io.open(filename, encoding='utf-8') as fd:
            return fd.read()

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    author="Jacob Rief",
    author_email="jacob.rief@gmail.com",
    name='djangoshop-stripe',
    version=shop_stripe.__version__,
    description='Stripe Payment Provider Integration for django-shop',
    long_description=convert('README.md', 'rst'),
    url='https://github.com/jrief/djangoshop-stripe',
    license='MIT License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'stripe>=1.53.0,<1.54',
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)
